<?php

declare(strict_types=1);

namespace Dedi\SyliusStockHistoryPlugin\DependencyInjection;

use Dedi\SyliusStockHistoryPlugin\Entity\InventoryLogEntry;
use Dedi\SyliusStockHistoryPlugin\Entity\InventoryLogEntryInterface;
use Dedi\SyliusStockHistoryPlugin\Factory\InventoryLogEntryFactory;
use Dedi\SyliusStockHistoryPlugin\Repository\InventoryLogEntryRepository;
use Sylius\Bundle\ResourceBundle\Controller\ResourceController;
use Symfony\Component\Config\Definition\Builder\ArrayNodeDefinition;
use Symfony\Component\Config\Definition\Builder\TreeBuilder;
use Symfony\Component\Config\Definition\ConfigurationInterface;

final class Configuration implements ConfigurationInterface
{
    /**
     * @psalm-suppress UnusedVariable
     */
    public function getConfigTreeBuilder(): TreeBuilder
    {
        $treeBuilder = new TreeBuilder('dedi_sylius_stock_history_plugin');
        $rootNode = $treeBuilder->getRootNode();

        $this->addResourcesSection($rootNode);

        return $treeBuilder;
    }

    private function addResourcesSection(ArrayNodeDefinition $node): void
    {
        $node
            ->children()
            ->arrayNode('resources')
                ->addDefaultsIfNotSet()
                ->children()
                    ->arrayNode('inventory_log_entry')
                        ->addDefaultsIfNotSet()
                        ->children()
                            ->variableNode('options')->end()
                            ->arrayNode('classes')
                                ->addDefaultsIfNotSet()
                                ->children()
                                    ->scalarNode('model')->defaultValue(InventoryLogEntry::class)->cannotBeEmpty()->end()
                                    ->scalarNode('interface')->defaultValue(InventoryLogEntryInterface::class)->cannotBeEmpty()->end()
                                    ->scalarNode('repository')->defaultValue(InventoryLogEntryRepository::class)->cannotBeEmpty()->end()
                                    ->scalarNode('factory')->defaultValue(InventoryLogEntryFactory::class)->end()
                                    ->scalarNode('controller')->defaultValue(ResourceController::class)->end()
                                ->end()
                            ->end()
                        ->end()
                    ->end()
                ->end()
            ->end()
        ->end()
        ;
    }
}
