<?php

declare(strict_types=1);

namespace Dedi\SyliusStockHistoryPlugin\Factory;

use DateTimeImmutable;
use Dedi\SyliusStockHistoryPlugin\Entity\InventoryLogEntry;
use Dedi\SyliusStockHistoryPlugin\Entity\InventoryLogEntryInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\ProductVariantInterface;

class InventoryLogEntryFactory implements InventoryLogEntryFactoryInterface
{
    public function createNew()
    {
        return new InventoryLogEntry();
    }

    public function create(
        ProductVariantInterface $variant,
        string $type,
        int $onHand,
        int $onHold,
        ?int $previousOnHand,
        ?int $previousOnHold,
        string $path,
        ?OrderInterface $order = null,
    ): InventoryLogEntryInterface {
        $inventoryLogEntry = $this->createNew();
        $inventoryLogEntry
            ->setLoggedAt(new DateTimeImmutable())
            ->setVariant($variant)
            ->setOrder($order)
            ->setInventoryName($variant->getInventoryName())
            ->setType($type)
            ->setOnHand($onHand)
            ->setOnHold($onHold)
            ->setPreviousOnHand($previousOnHand)
            ->setPreviousOnHold($previousOnHold)
            ->setPath($path)
        ;

        return $inventoryLogEntry;
    }
}
